@@ -24,36 +24,39 @@
 # Module Initialization
 ############################################################################
 
-use diagnostics;
 use strict;
 
 package Attachment;
 
-# Use the template toolkit (http://www.template-toolkit.org/) to generate
-# the user interface (HTML pages and mail messages) using templates in the
-# "templates/" subdirectory.
-use Template;
-
-# This is the global template object that gets used one or more times by
-# the script when it needs to process a template and return the results.
-# Configuration parameters can be specified here that apply to all templates
-# processed in this file.
-my $template = Template->new(
-  {
-    # Colon-separated list of directories containing templates.
-    INCLUDE_PATH => 'template/custom:/usr/share/bugzilla/lib/template/default' ,
-    # Allow templates to be specified with relative paths.
-    RELATIVE => 1 
-  }
-);
-
 # This module requires that its caller have said "require CGI.pl" to import
 # relevant functions from that script and its companion globals.pl.
 
+# Use the Flag module to handle flags.
+use Bugzilla::Flag;
+
 ############################################################################
 # Functions
 ############################################################################
 
+sub new {
+    # Returns a hash of information about the attachment with the given ID.
+
+    my ($invocant, $id) = @_;
+    return undef if !$id;
+    my $self = { 'id' => $id };
+    my $class = ref($invocant) || $invocant;
+    bless($self, $class);
+    
+    &::PushGlobalSQLState();
+    &::SendSQL("SELECT 1, description, bug_id FROM attachments " . 
+               "WHERE attach_id = $id");
+    ($self->{'exists'}, $self->{'summary'}, $self->{'bug_id'}) = 
+      &::FetchSQLData();
+    &::PopGlobalSQLState();
+
+    return $self;
+}
+
 sub query
 {
   # Retrieves and returns an array of attachment records for a given bug. 
@@ -67,7 +70,7 @@ sub query
   # of hashes in which each hash represents a single attachment.
   &::SendSQL("
               SELECT attach_id, creation_ts, mimetype, description, ispatch, 
-               isobsolete, submitter_id 
+               isobsolete, isprivate, submitter_id
               FROM attachments WHERE bug_id = $bugid ORDER BY attach_id
             ");
   my @attachments = ();
@@ -75,7 +78,8 @@ sub query
     my %a;
     my $submitter_id;
     ($a{'attachid'}, $a{'date'}, $a{'contenttype'}, $a{'description'},
-     $a{'ispatch'}, $a{'isobsolete'}, $submitter_id) = &::FetchSQLData();
+     $a{'ispatch'}, $a{'isobsolete'}, $a{'isprivate'}, $submitter_id) 
+        = &::FetchSQLData();
 
     # Format the attachment's creation/modification date into a standard
     # format (YYYY-MM-DD HH:MM)
@@ -83,23 +87,9 @@ sub query
         $a{'date'} = "$1-$2-$3 $4:$5";
     }
 
-    # Retrieve a list of status flags that have been set on the attachment.
-    &::PushGlobalSQLState();
-    &::SendSQL(" 
-                SELECT   name 
-                FROM     attachstatuses, attachstatusdefs 
-                WHERE    attach_id = $a{'attachid'} 
-                AND      attachstatuses.statusid = attachstatusdefs.id
-                ORDER BY sortkey
-              ");
-    my @statuses = ();
-    while (&::MoreSQLData()) { 
-      my ($status) = &::FetchSQLData(); 
-      push @statuses , $status;
-    }
-    $a{'statuses'} = \@statuses;
-    &::PopGlobalSQLState();
-
+    # Retrieve a list of flags for this attachment.
+    $a{'flags'} = Bugzilla::Flag::match({ 'attach_id' => $a{'attachid'} });
+    
     # We will display the edit link if the user can edit the attachment;
     # ie the are the submitter, or they have canedit.
     # Also show the link if the user is not logged in - in that cae,
