@@ -21,26 +21,24 @@
 #                 Terry Weissman <terry@mozilla.org>
 #                 Chris Yeh      <cyeh@bluemartini.com>
 
-use diagnostics;
 use strict;
 
 use DBI;
-
-use lib '/usr/share/bugzilla/lib';
-
 use RelationSet;
 use vars qw($unconfirmedstate $legal_keywords);
-require "globals.pl";
-require "CGI.pl";
+
 package Bug;
 use CGI::Carp qw(fatalsToBrowser);
 my %ok_field;
 
-for my $key (qw (bug_id product version rep_platform op_sys bug_status 
+use Bugzilla::Config;
+use Bugzilla::Util;
+
+for my $key (qw (bug_id alias product version rep_platform op_sys bug_status 
                 resolution priority bug_severity component assigned_to
                 reporter bug_file_loc short_desc target_milestone 
-                qa_contact status_whiteboard creation_ts groupset 
-                delta_ts votes whoid usergroupset comment query error) ){
+                qa_contact status_whiteboard creation_ts 
+                delta_ts votes whoid comment query error) ){
     $ok_field{$key}++;
     }
 
@@ -77,9 +75,12 @@ sub initBug  {
   my $self = shift();
   my ($bug_id, $user_id) = (@_);
 
+  # If the bug ID isn't numeric, it might be an alias, so try to convert it.
+  $bug_id = &::BugAliasToID($bug_id) if $bug_id !~ /^[1-9][0-9]*$/;
+  
   my $old_bug_id = $bug_id;
-  if ((! defined $bug_id) || (!$bug_id) || (!&::detaint_natural($bug_id))) {
-      # no bug number given
+  if ((! defined $bug_id) || (!$bug_id) || (!detaint_natural($bug_id))) {
+      # no bug number given or the alias didn't match a bug
       $self->{'bug_id'} = $old_bug_id;
       $self->{'error'} = "InvalidBugId";
       return $self;
@@ -95,60 +96,48 @@ sub initBug  {
      }
   }
      
-
-  &::ConnectToDatabase();
-  &::GetVersionTable();
-
-  # this verification should already have been done by caller
-  # my $loginok = quietly_check_login();
-
-
   $self->{'whoid'} = $user_id;
-  &::SendSQL("SELECT groupset FROM profiles WHERE userid=$self->{'whoid'}");
-  my $usergroupset = &::FetchOneColumn();
-  if (!$usergroupset) { $usergroupset = '0' }
-  $self->{'usergroupset'} = $usergroupset;
 
   my $query = "
     select
-      bugs.bug_id, product, version, rep_platform, op_sys, bug_status,
-      resolution, priority, bug_severity, component, assigned_to, reporter,
+      bugs.bug_id, alias, products.name, version, rep_platform, op_sys, bug_status,
+      resolution, priority, bug_severity, components.name, assigned_to, reporter,
       bug_file_loc, short_desc, target_milestone, qa_contact,
       status_whiteboard, date_format(creation_ts,'%Y-%m-%d %H:%i'),
-      groupset, delta_ts, ifnull(sum(votes.count),0)
-    from bugs left join votes using(bug_id)
+      delta_ts, sum(votes.count)
+    from bugs left join votes using(bug_id),
+      products, components
     where bugs.bug_id = $bug_id
+      AND products.id = bugs.product_id
+      AND components.id = bugs.component_id
     group by bugs.bug_id";
 
-  &::SendSQL(&::SelectVisible($query, $user_id, $usergroupset));
-  my @row;
+  &::SendSQL($query);
+  my @row = ();
 
-  if (@row = &::FetchSQLData()) {
+  if ((@row = &::FetchSQLData()) && &::CanSeeBug($bug_id, $self->{'whoid'})) {
     my $count = 0;
     my %fields;
-    foreach my $field ("bug_id", "product", "version", "rep_platform",
+    foreach my $field ("bug_id", "alias", "product", "version", "rep_platform",
                        "op_sys", "bug_status", "resolution", "priority",
                        "bug_severity", "component", "assigned_to", "reporter",
                        "bug_file_loc", "short_desc", "target_milestone",
                        "qa_contact", "status_whiteboard", "creation_ts",
-                       "groupset", "delta_ts", "votes") {
+                       "delta_ts", "votes") {
         $fields{$field} = shift @row;
         if ($fields{$field}) {
             $self->{$field} = $fields{$field};
         }
         $count++;
     }
-  } else {
-    &::SendSQL("select groupset from bugs where bug_id = $bug_id");
-    if (@row = &::FetchSQLData()) {
+  } elsif (@row) {
       $self->{'bug_id'} = $bug_id;
       $self->{'error'} = "NotPermitted";
       return $self;
-    } else {
+  } else {
       $self->{'bug_id'} = $bug_id;
       $self->{'error'} = "NotFound";
       return $self;
-    }
   }
 
   $self->{'assigned_to'} = &::DBID_to_name($self->{'assigned_to'});
@@ -161,7 +150,7 @@ sub initBug  {
     $self->{'cc'} = \@cc;
   }
 
-  if (&::Param("useqacontact") && (defined $self->{'qa_contact'}) ) {
+  if (Param("useqacontact") && (defined $self->{'qa_contact'}) ) {
     my $name = $self->{'qa_contact'} > 0 ? &::DBID_to_name($self->{'qa_contact'}) :"";
     if ($name) {
       $self->{'qa_contact'} = $name;
@@ -183,50 +172,47 @@ sub initBug  {
     }
   }
 
-  &::SendSQL("select attach_id, creation_ts, description 
-           from attachments 
-           where bug_id = $bug_id");
-  my @attachments;
-  while (&::MoreSQLData()) {
-    my ($attachid, $date, $desc) = (&::FetchSQLData());
-    if ($date =~ /^(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
-        $date = "$3/$4/$2 $5:$6";
-      my %attach;
-      $attach{'attachid'} = $attachid;
-      $attach{'date'} = $date;
-      $attach{'desc'} = $desc;
-      push @attachments, \%attach;
+    &::SendSQL("select attach_id, creation_ts, isprivate, description 
+             from attachments 
+             where bug_id = $bug_id");
+    my @attachments;
+    while (&::MoreSQLData()) {
+        my ($attachid, $date, $isprivate, $desc) = (&::FetchSQLData());
+        my %attach;
+        $attach{'attachid'} = $attachid;
+        $attach{'isprivate'} = $isprivate;
+        $attach{'date'} = $date;
+        $attach{'desc'} = $desc;
+        push @attachments, \%attach;
+    }
+    if (@attachments) {
+        $self->{'attachments'} = \@attachments;
     }
-  }
-  if (@attachments) {
-    $self->{'attachments'} = \@attachments;
-  }
 
-  &::SendSQL("select bug_id, who, bug_when, thetext 
+    &::SendSQL("select bug_id, who, bug_when, isprivate, thetext 
            from longdescs 
            where bug_id = $bug_id");
-  my @longdescs;
-  while (&::MoreSQLData()) {
-    my ($bug_id, $who, $bug_when, $thetext) = (&::FetchSQLData());
-    my %longdesc;
-    $longdesc{'who'} = $who;
-    $longdesc{'bug_when'} = $bug_when;
-    $longdesc{'thetext'} = $thetext;
-    push @longdescs, \%longdesc;
-  }
-  if (@longdescs) {
-    $self->{'longdescs'} = \@longdescs;
-  }
-
-  if (&::Param("usedependencies")) {
-    my @depends = EmitDependList("blocked", "dependson", $bug_id);
-    if ( @depends ) {
+    my @longdescs;
+    while (&::MoreSQLData()) {
+        my ($bug_id, $who, $bug_when, $isprivate, $thetext) = (&::FetchSQLData());
+        my %longdesc;
+        $longdesc{'who'} = $who;
+        $longdesc{'bug_when'} = $bug_when;
+        $longdesc{'isprivate'} = $isprivate;
+        $longdesc{'thetext'} = $thetext;
+        push @longdescs, \%longdesc;
+    }
+    if (@longdescs) {
+        $self->{'longdescs'} = \@longdescs;
+    }
+  
+  my @depends = EmitDependList("blocked", "dependson", $bug_id);
+  if (@depends) {
       $self->{'dependson'} = \@depends;
-    }
-    my @blocks = EmitDependList("dependson", "blocked", $bug_id);
-    if ( @blocks ) {
-      $self->{'blocks'} = \@blocks;
-    }
+  }  
+  my @blocks = EmitDependList("dependson", "blocked", $bug_id);
+  if (@blocks) {
+    $self->{'blocks'} = \@blocks;
   }
 
   return $self;
@@ -251,7 +237,7 @@ sub emitXML {
 
   $xml .= "<bug>\n";
 
-  foreach my $field ("bug_id", "bug_status", "product",
+  foreach my $field ("bug_id", "alias", "bug_status", "product",
       "priority", "version", "rep_platform", "assigned_to", "delta_ts", 
       "component", "reporter", "target_milestone", "bug_severity", 
       "creation_ts", "qa_contact", "op_sys", "resolution", "bug_file_loc",
@@ -269,34 +255,40 @@ sub emitXML {
     }
   }
 
-  if (defined $self->{'longdescs'}) {
-    for (my $i=0 ; $i < @{$self->{'longdescs'}} ; $i++) {
-      $xml .= "  <long_desc>\n"; 
-      $xml .= "   <who>" . &::DBID_to_name($self->{'longdescs'}[$i]->{'who'}) 
-                         . "</who>\n"; 
-      $xml .= "   <bug_when>" . $self->{'longdescs'}[$i]->{'bug_when'} 
-                              . "</bug_when>\n"; 
-      $xml .= "   <thetext>" . QuoteXMLChars($self->{'longdescs'}[$i]->{'thetext'})
-                             . "</thetext>\n"; 
-      $xml .= "  </long_desc>\n"; 
+    if (defined $self->{'longdescs'}) {
+        for (my $i=0 ; $i < @{$self->{'longdescs'}} ; $i++) {
+            next if ($self->{'longdescs'}[$i]->{'isprivate'} 
+                     && Param("insidergroup")
+                     && !&::UserInGroup(Param("insidergroup")));
+            $xml .= "  <long_desc>\n"; 
+            $xml .= "   <who>" . &::DBID_to_name($self->{'longdescs'}[$i]->{'who'}) 
+                               . "</who>\n"; 
+            $xml .= "   <bug_when>" . $self->{'longdescs'}[$i]->{'bug_when'} 
+                                    . "</bug_when>\n"; 
+            $xml .= "   <thetext>" . QuoteXMLChars($self->{'longdescs'}[$i]->{'thetext'})
+                                   . "</thetext>\n"; 
+            $xml .= "  </long_desc>\n"; 
+        }
     }
-  }
 
-  if (defined $self->{'attachments'}) {
-    for (my $i=0 ; $i < @{$self->{'attachments'}} ; $i++) {
-      $xml .= "  <attachment>\n"; 
-      $xml .= "    <attachid>" . $self->{'attachments'}[$i]->{'attachid'}
-                              . "</attachid>\n"; 
-      $xml .= "    <date>" . $self->{'attachments'}[$i]->{'date'} . "</date>\n"; 
-      $xml .= "    <desc>" . QuoteXMLChars($self->{'attachments'}[$i]->{'desc'}) . "</desc>\n"; 
-    # $xml .= "    <type>" . $self->{'attachments'}[$i]->{'type'} . "</type>\n"; 
-    # $xml .= "    <data>" . $self->{'attachments'}[$i]->{'data'} . "</data>\n"; 
-      $xml .= "  </attachment>\n"; 
+    if (defined $self->{'attachments'}) {
+        for (my $i=0 ; $i < @{$self->{'attachments'}} ; $i++) {
+            next if ($self->{'attachments'}[$i]->{'isprivate'} 
+                     && Param("insidergroup")
+                     && !&::UserInGroup(Param("insidergroup")));
+            $xml .= "  <attachment>\n"; 
+            $xml .= "    <attachid>" . $self->{'attachments'}[$i]->{'attachid'}
+                                    . "</attachid>\n"; 
+            $xml .= "    <date>" . $self->{'attachments'}[$i]->{'date'} . "</date>\n"; 
+            $xml .= "    <desc>" . QuoteXMLChars($self->{'attachments'}[$i]->{'desc'}) . "</desc>\n"; 
+          # $xml .= "    <type>" . $self->{'attachments'}[$i]->{'type'} . "</type>\n"; 
+          # $xml .= "    <data>" . $self->{'attachments'}[$i]->{'data'} . "</data>\n"; 
+            $xml .= "  </attachment>\n"; 
+        }
     }
-  }
 
-  $xml .= "</bug>\n";
-  return $xml;
+    $xml .= "</bug>\n";
+    return $xml;
 }
 
 sub EmitDependList {
@@ -329,9 +321,7 @@ sub XML_Header {
 
   my $xml;
   $xml = "<?xml version=\"1.0\" standalone=\"yes\"?>\n";
-  my $no_cgi_urlbase = $urlbase;
-  $no_cgi_urlbase =~ s{/cgi-bin}{};
-  $xml .= "<!DOCTYPE bugzilla SYSTEM \"$no_cgi_urlbase";
+  $xml .= "<!DOCTYPE bugzilla SYSTEM \"$urlbase";
   if (! ($urlbase =~ /.+\/$/)) {
     $xml .= "/";
   }
@@ -351,22 +341,6 @@ sub XML_Footer {
   return ("</bugzilla>\n");
 }
 
-sub UserInGroup {
-    my $self = shift();
-    my ($groupname) = (@_);
-    if ($self->{'usergroupset'} eq "0") {
-        return 0;
-    }
-    &::ConnectToDatabase();
-    &::SendSQL("select (bit & $self->{'usergroupset'}) != 0 from groups where name = " 
-           . &::SqlQuote($groupname));
-    my $bit = &::FetchOneColumn();
-    if ($bit) {
-        return 1;
-    }
-    return 0;
-}
-
 sub CanChangeField {
    my $self = shift();
    my ($f, $oldvalue, $newvalue) = (@_);
@@ -388,7 +362,7 @@ sub CanChangeField {
     if ($oldvalue eq $newvalue) {
         return 1;
     }
-    if (&::trim($oldvalue) eq &::trim($newvalue)) {
+    if (trim($oldvalue) eq trim($newvalue)) {
         return 1;
     }
     if ($f =~ /^longdesc/) {
