@@ -24,12 +24,13 @@
 ################################################################################
 
 # Make it harder for us to do dangerous things in Perl.
-use diagnostics;
 use strict;
 
 # Bundle the functions in this file together into the "Token" package.
 package Token;
 
+use Bugzilla::Config;
+
 use Date::Format;
 
 # This module requires that its caller have said "require CGI.pl" to import
@@ -76,14 +77,14 @@ sub IssueEmailChangeToken {
     my $template = $::template;
     my $vars = $::vars;
 
-    $vars->{'oldemailaddress'} = $old_email . &::Param('emailsuffix');
-    $vars->{'newemailaddress'} = $new_email . &::Param('emailsuffix');
+    $vars->{'oldemailaddress'} = $old_email . Param('emailsuffix');
+    $vars->{'newemailaddress'} = $new_email . Param('emailsuffix');
     
     $vars->{'max_token_age'} = $maxtokenage;
     $vars->{'token_ts'} = $token_ts;
 
     $vars->{'token'} = $token;
-    $vars->{'emailaddress'} = $old_email . &::Param('emailsuffix');
+    $vars->{'emailaddress'} = $old_email . Param('emailsuffix');
 
     my $message;
     $template->process("account/email/change-old.txt.tmpl", $vars, \$message)
@@ -94,7 +95,7 @@ sub IssueEmailChangeToken {
     close SENDMAIL;
 
     $vars->{'token'} = $newtoken;
-    $vars->{'emailaddress'} = $new_email . &::Param('emailsuffix');
+    $vars->{'emailaddress'} = $new_email . Param('emailsuffix');
 
     $message = "";
     $template->process("account/email/change-new.txt.tmpl", $vars, \$message)
@@ -114,19 +115,11 @@ sub IssuePasswordToken {
 
     # Retrieve the user's ID from the database.
     my $quotedloginname = &::SqlQuote($loginname);
-    &::SendSQL("SELECT profiles.userid, tokens.issuedate FROM profiles 
-                    LEFT JOIN tokens
-                    ON tokens.userid = profiles.userid
-                    AND tokens.tokentype = 'password'
-                    AND tokens.issuedate > DATE_SUB(NOW(), INTERVAL 10 MINUTE)
-                    WHERE login_name = $quotedloginname");
-    my ($userid, $toosoon) = &::FetchSQLData();
-
-    if ($toosoon) {
-        &::DisplayError('Too Soon For Another Password Token') && exit;
-    };
+    &::SendSQL("SELECT userid FROM profiles WHERE login_name = $quotedloginname");
+    my ($userid) = &::FetchSQLData();
 
     my $token_ts = time();
+    my $issuedate = time2str("%Y-%m-%d %H:%M", $token_ts);
 
     # Generate a unique token and insert it into the tokens table.
     # We have to lock the tokens table before generating the token, 
@@ -136,7 +129,7 @@ sub IssuePasswordToken {
     my $quotedtoken = &::SqlQuote($token);
     my $quotedipaddr = &::SqlQuote($::ENV{'REMOTE_ADDR'});
     &::SendSQL("INSERT INTO tokens ( userid , issuedate , token , tokentype , eventdata )
-                VALUES      ( $userid , NOW() , $quotedtoken , 'password' , $quotedipaddr )");
+                VALUES      ( $userid , '$issuedate' , $quotedtoken , 'password' , $quotedipaddr )");
     &::SendSQL("UNLOCK TABLES");
 
     # Mail the user the token along with instructions for using it.
@@ -145,7 +138,7 @@ sub IssuePasswordToken {
     my $vars = $::vars;
 
     $vars->{'token'} = $token;
-    $vars->{'emailaddress'} = $loginname . &::Param('emailsuffix');
+    $vars->{'emailaddress'} = $loginname . Param('emailsuffix');
 
     $vars->{'max_token_age'} = $maxtokenage;
     $vars->{'token_ts'} = $token_ts;
@@ -183,8 +176,7 @@ sub GenerateUniqueToken {
 
         ++$tries;
         if ($tries > 100) {
-            &::DisplayError("Something is seriously wrong with the token generation system.");
-            exit;
+            &::ThrowCodeError("token_generation_error");
         }
 
         $token = &::GenerateRandomPassword();
@@ -215,7 +207,7 @@ sub Cancel {
     my ($issuedate, $tokentype, $eventdata, $loginname, $realname) = &::FetchSQLData();
 
     # Get the email address of the Bugzilla maintainer.
-    my $maintainer = &::Param('maintainer');
+    my $maintainer = Param('maintainer');
 
     # Format the user's real name and email address into a single string.
     my $username = $realname ? $realname . " <" . $loginname . ">" : $loginname;
