@@ -19,8 +19,8 @@
 #
 # Contributor(s): Terry Weissman <terry@mozilla.org>
 #                 Dave Miller <justdave@syndicomm.com>
+#                 Vaskin Kissoyan <vkissoyan@yahoo.com>
 
-use diagnostics;
 use strict;
 
 use RelationSet;
@@ -28,6 +28,10 @@ use RelationSet;
 # Use the Attachment module to display attachments for the bug.
 use Attachment;
 
+# Use the Flag modules to display flags on the bug.
+use Bugzilla::Flag;
+use Bugzilla::FlagType;
+
 sub show_bug {    
     # Shut up misguided -w warnings about "used only once".  For some reason,
     # "use vars" chokes on me when I try it here.
@@ -58,7 +62,6 @@ sub show_bug {
     my $vars = $::vars;
     
     $vars->{'GetBugLink'} = \&GetBugLink;
-    $vars->{'quoteUrls'} = \&quoteUrls,
     $vars->{'lsearch'} = \&lsearch,
     $vars->{'header_done'} = (@_),
 
@@ -77,29 +80,50 @@ sub show_bug {
 
     # Populate the bug hash with the info we get directly from the DB.
     my $query = "
-    SELECT bugs.bug_id, product, version, rep_platform, 
-        op_sys, bug_status, resolution, priority, 
-        bug_severity, component, assigned_to, reporter, 
-        bug_file_loc, short_desc, target_milestone, 
+    SELECT bugs.bug_id, alias, bugs.product_id, products.name, version, 
+        rep_platform, op_sys, bug_status, resolution, priority, 
+        bug_severity, bugs.component_id, components.name, assigned_to, 
+        reporter, bug_file_loc, short_desc, target_milestone, 
         qa_contact, status_whiteboard, 
         date_format(creation_ts,'%Y-%m-%d %H:%i'),
-        groupset, delta_ts, ifnull(sum(votes.count),0)
-    FROM bugs LEFT JOIN votes USING(bug_id)
+        delta_ts, sum(votes.count), delta_ts calc_disp_date,
+        estimated_time, remaining_time
+    FROM bugs LEFT JOIN votes USING(bug_id), products, components
     WHERE bugs.bug_id = $id
+        AND bugs.product_id = products.id
+        AND bugs.component_id = components.id
     GROUP BY bugs.bug_id";
 
     SendSQL($query);
 
+    # The caller is meant to have checked this. Abort here so that
+    # we don't get obscure SQL errors, below
+    if (!MoreSQLData()) {
+        $vars->{'bug_id'} = $id;
+        ThrowCodeError("no_bug_data");
+    }
+
     my $value;
+    my $disp_date;
     my @row = FetchSQLData();
-    foreach my $field ("bug_id", "product", "version", "rep_platform",
-                       "op_sys", "bug_status", "resolution", "priority",
-                       "bug_severity", "component", "assigned_to", "reporter",
-                       "bug_file_loc", "short_desc", "target_milestone",
-                       "qa_contact", "status_whiteboard", "creation_ts",
-                       "groupset", "delta_ts", "votes") 
+    foreach my $field ("bug_id", "alias", "product_id", "product", "version", 
+                       "rep_platform", "op_sys", "bug_status", "resolution", 
+                       "priority", "bug_severity", "component_id", "component", 
+                       "assigned_to", "reporter", "bug_file_loc", "short_desc", 
+                       "target_milestone", "qa_contact", "status_whiteboard", 
+                       "creation_ts", "delta_ts", "votes", "calc_disp_date", 
+                       "estimated_time", "remaining_time")
     {
         $value = shift(@row);
+        if ($field eq "calc_disp_date") {
+            # Convert MySQL timestamp (_ts) to datetime format(%Y-%m-%d %H:%i)
+            $disp_date = substr($value,0,4) . '-';
+            $disp_date .= substr($value,4,2) . '-';
+            $disp_date .= substr($value,6,2) . ' ';
+            $disp_date .= substr($value,8,2) . ':';
+            $disp_date .= substr($value,10,2);
+            $value = $disp_date; 
+        }
         $bug{$field} = defined($value) ? $value : "";
     }
 
@@ -179,6 +203,37 @@ sub show_bug {
 
     # Attachments
     $bug{'attachments'} = Attachment::query($id);
+   
+    # The types of flags that can be set on this bug.
+    # If none, no UI for setting flags will be displayed.
+    my $flag_types = 
+      Bugzilla::FlagType::match({ 'target_type'  => 'bug', 
+                                  'product_id'   => $bug{'product_id'}, 
+                                  'component_id' => $bug{'component_id'} });
+    foreach my $flag_type (@$flag_types) {
+        $flag_type->{'flags'} = 
+          Bugzilla::Flag::match({ 'bug_id'      => $id , 
+                                  'type_id'     => $flag_type->{'id'} , 
+                                  'target_type' => 'bug' });
+    }
+    $vars->{'flag_types'} = $flag_types;
+    $vars->{'any_flags_requesteeble'} = grep($_->{'is_requesteeble'}, @$flag_types);
+
+    # The number of types of flags that can be set on attachments to this bug
+    # and the number of flags on those attachments.  One of these counts must be
+    # greater than zero in order for the "flags" column to appear in the table 
+    # of attachments.
+    my $num_attachment_flag_types = 
+      Bugzilla::FlagType::count({ 'target_type'  => 'attachment', 
+                                  'product_id'   => $bug{'product_id'}, 
+                                  'component_id' => $bug{'component_id'},
+                                  'is_active'    => 1 });
+    my $num_attachment_flags = 
+      Bugzilla::Flag::count({ 'target_type'  => 'attachment', 
+                              'bug_id'       => $id });
+    
+    $vars->{'show_attachment_flags'} 
+      = $num_attachment_flag_types || $num_attachment_flags;
 
     # Dependencies
     my @list;
@@ -189,6 +244,14 @@ sub show_bug {
         push(@list, $i);
     }
 
+    if (UserInGroup(Param("timetrackinggroup"))) {
+
+        SendSQL("SELECT SUM(work_time) 
+                 FROM longdescs WHERE longdescs.bug_id=$id");
+        $bug{'actual_time'} = FetchSQLData();
+
+    }
+
     $bug{'dependson'} = \@list;
 
     my @list2;
@@ -203,65 +266,75 @@ sub show_bug {
 
     # Groups
     my @groups;
-    if ($::usergroupset ne '0' || $bug{'groupset'} ne '0') {      
-        my $bug_groupset = $bug{'groupset'};
 
-        SendSQL("SELECT bit, name, description, (bit & $bug_groupset != 0),
-                 (bit & $::usergroupset != 0) FROM groups 
-                 WHERE isbuggroup != 0 " .
-                 # Include active groups as well as inactive groups to which
-                 # the bug already belongs.  This way the bug can be removed
-                 # from an inactive group but can only be added to active ones.
-                "AND ((isactive = 1 AND (bit & $::usergroupset != 0)) OR
-                 (bit & $bug_groupset != 0))");
+    # For every group, we need to know if there is ANY bug_group_map
+    # record putting the current bug in that group and if there is ANY
+    # user_group_map record putting the user in that group.
+    # The LEFT JOINs are checking for record existence.
+    #
+    SendSQL("SELECT DISTINCT groups.id, name, description," .
+             " bug_group_map.group_id IS NOT NULL," .
+             " user_group_map.group_id IS NOT NULL," .
+             " isactive" .
+             " FROM groups" . 
+             " LEFT JOIN bug_group_map" .
+             " ON bug_group_map.group_id = groups.id" .
+             " AND bug_id = $bug{'bug_id'}" .
+             " LEFT JOIN user_group_map" .
+             " ON user_group_map.group_id = groups.id" .
+             " AND user_id = $::userid" .
+             " AND NOT isbless" .
+             " WHERE isbuggroup");
 
-        $user{'inallgroups'} = 1;
+    $user{'inallgroups'} = 1;
 
-        while (MoreSQLData()) {
-            my ($bit, $name, $description, $ison, $ingroup) = FetchSQLData();
-            # For product groups, we only want to display the checkbox if either
-            # (1) The bit is already set, or
-            # (2) The user is in the group, but either:
-            #     (a) The group is a product group for the current product, or
-            #     (b) The group name isn't a product name
-            # This means that all product groups will be skipped, but 
-            # non-product bug groups will still be displayed.
-            if($ison || 
-               ($ingroup && (($name eq $bug{'product'}) ||
-                             (!defined $::proddesc{$name}))))
-            {
-                $user{'inallgroups'} &= $ingroup;
+    while (MoreSQLData()) {
+        my ($groupid, $name, $description, $ison, $ingroup, $isactive) 
+            = FetchSQLData();
 
-                push (@groups, { "bit" => $bit,
-                                 "ison" => $ison,
-                                 "ingroup" => $ingroup,
-                                 "description" => $description });            
-            }
+        $bug{'inagroup'} = 1 if ($ison);
+
+        # For product groups, we only want to display the checkbox if either
+        # (1) The bit is already set, or
+        # (2) The user is in the group, but either:
+        #     (a) The group is a product group for the current product, or
+        #     (b) The group name isn't a product name
+        # This means that all product groups will be skipped, but 
+        # non-product bug groups will still be displayed.
+        if($ison || 
+           ($isactive && ($ingroup && (!Param("usebuggroups") || ($name eq $bug{'product'}) ||
+                         (!defined $::proddesc{$name})))))
+        {
+            $user{'inallgroups'} &= $ingroup;
+
+            push (@groups, { "bit" => $groupid,
+                             "ison" => $ison,
+                             "ingroup" => $ingroup,
+                             "description" => $description });            
         }
+    }
 
-        # If the bug is restricted to a group, display checkboxes that allow
-        # the user to set whether or not the reporter 
-        # and cc list can see the bug even if they are not members of all 
-        # groups to which the bug is restricted.
-        if ($bug{'groupset'} != 0) {
-            $bug{'inagroup'} = 1;
+    # If the bug is restricted to a group, get flags that allow
+    # the user to set whether or not the reporter 
+    # and cc list can see the bug even if they are not members of all 
+    # groups to which the bug is restricted.
+    if ($bug{'inagroup'}) {
 
-            # Determine whether or not the bug is always accessible by the
-            # reporter, QA contact, and/or users on the cc: list.
-            SendSQL("SELECT reporter_accessible, cclist_accessible
-                     FROM   bugs
-                     WHERE  bug_id = $id
-                    ");
-            ($bug{'reporter_accessible'}, 
-             $bug{'cclist_accessible'}) = FetchSQLData();        
-        }
+        # Determine whether or not the bug is always accessible by the
+        # reporter, QA contact, and/or users on the cc: list.
+        SendSQL("SELECT reporter_accessible, cclist_accessible
+                 FROM   bugs
+                 WHERE  bug_id = $id
+                ");
+        ($bug{'reporter_accessible'}, 
+         $bug{'cclist_accessible'}) = FetchSQLData();        
     }
     $vars->{'groups'} = \@groups;
 
     my $movers = Param("movers");
     $user{'canmove'} = Param("move-enabled") 
                        && (defined $::COOKIE{"Bugzilla_login"}) 
-                       && ($::COOKIE{"Bugzilla_login"} =~ /\Q$movers\E/);
+                       && ($::COOKIE{"Bugzilla_login"} =~ /$movers/);
 
     # User permissions
 
@@ -314,9 +387,6 @@ sub show_bug {
     $vars->{'bug'} = \%bug;
     $vars->{'user'} = \%user;
 
-    # Create the <link> elements for browsing bug lists
-    $vars->{'navigation_links'} = navigation_links(join(':',@bug_list));
-
     # Generate and return the UI (HTML page) from the appropriate template.
     $template->process("bug/edit.html.tmpl", $vars)
       || ThrowTemplateError($template->error());
