@@ -26,7 +26,6 @@
 
 
 use AnyDBM_File;
-use diagnostics;
 use strict;
 use vars @::legal_product;
 
@@ -54,10 +53,27 @@ foreach (@myproducts) {
 
 &calculate_dupes();
 
+# Generate a static RDF file containing the default view of the duplicates data.
+open(CGI, "REQUEST_METHOD=GET QUERY_STRING=ctype=rdf ./duplicates.cgi |")
+  || die "can't fork duplicates.cgi: $!";
+open(RDF, ">data/duplicates.tmp")
+  || die "can't write to data/duplicates.tmp: $!";
+my $headers_done = 0;
+while (<CGI>) {
+  print RDF if $headers_done;
+  $headers_done = 1 if $_ eq "\n";
+}
+close CGI;
+close RDF;
+if (-s "data/duplicates.tmp") {
+    rename("data/duplicates.rdf", "data/duplicates-old.rdf");
+    rename("data/duplicates.tmp", "data/duplicates.rdf");
+}
+
 sub check_data_dir {
     my $dir = shift;
 
-    if (! -d $dir) {
+    if (! -d) {
         mkdir $dir, 0777;
         chmod 0777, $dir;
     }
@@ -67,6 +83,9 @@ sub collect_stats {
     my $dir = shift;
     my $product = shift;
     my $when = localtime (time);
+    my $product_id = get_product_id($product) unless $product eq '-All-';
+
+    die "Unknown product $product" unless ($product_id or $product eq '-All-');
 
     # NB: Need to mangle the product for the filename, but use the real
     # product name in the query
@@ -82,7 +101,7 @@ sub collect_stats {
             if( $product eq "-All-" ) {
                 SendSQL("select count(bug_status) from bugs where bug_status='$status'");
             } else {
-                SendSQL("select count(bug_status) from bugs where bug_status='$status' and product=" . SqlQuote($product));
+                SendSQL("select count(bug_status) from bugs where bug_status='$status' and product_id=$product_id");
             }
 
             push @row, FetchOneColumn();
@@ -92,7 +111,7 @@ sub collect_stats {
             if( $product eq "-All-" ) {
                 SendSQL("select count(resolution) from bugs where resolution='$resolution'");
             } else {
-                SendSQL("select count(resolution) from bugs where resolution='$resolution' and product=" . SqlQuote($product));
+                SendSQL("select count(resolution) from bugs where resolution='$resolution' and product_id=$product_id");
             }
 
             push @row, FetchOneColumn();
