@@ -25,7 +25,6 @@
 # a new bug into bugzilla. Everything before the beginning <?xml line
 # is removed so you can pipe in email messages.
 
-use diagnostics;
 use strict;
 
 #####################################################################
@@ -75,14 +74,12 @@ GetVersionTable();
 sub sillyness {
     my $zz;
     $zz = $Data::Dumper::Useqq;
-    $zz = %::components;
     $zz = %::versions;
     $zz = %::keywordsbyname;
     $zz = @::legal_bug_status;
     $zz = @::legal_opsys;
     $zz = @::legal_platform;
     $zz = @::legal_priority;
-    $zz = @::legal_product;
     $zz = @::legal_severity;
     $zz = @::legal_resolution;
     $zz = %::target_milestone;
@@ -229,7 +226,7 @@ if (defined $tree->[1][0]->{'urlbase'}) {
 }
   
 
-my $bugqty = ($#{$tree->[1]} +1 -3) / 4;
+my $bugqty = ($#{@{$tree}->[1]} +1 -3) / 4;
 my $log = "Imported $bugqty bug(s) from $urlbase,\n  sent by $exporter.\n\n";
 for (my $k=1 ; $k <= $bugqty ; $k++) {
   my $cur = $k*4;
@@ -263,7 +260,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
  
   my %bug_fields;
   my $err = "";
-  for (my $i=3 ; $i < $#{$tree->[1][$cur]} ; $i=$i+4) {
+  for (my $i=3 ; $i < $#{@{$tree}->[1][$cur]} ; $i=$i+4) {
     if (defined $multiple_fields{$tree->[1][$cur][$i]}) {
       if (defined $bug_fields{$tree->[1][$cur][$i]}) {
         $bug_fields{$tree->[1][$cur][$i]} .= " " .  $tree->[1][$cur][$i+1][2];
@@ -279,7 +276,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
       $err .= "<$tree->[1][$cur][$i]>";
       if (defined $tree->[1][$cur][$i+1][3]) {
         $err .= "\n";
-        for (my $j=3 ; $j < $#{$tree->[1][$cur][$i+1]} ; $j=$j+4) {
+        for (my $j=3 ; $j < $#{@{$tree}->[1][$cur][$i+1]} ; $j=$j+4) {
           $err .= "  <". $tree->[1][$cur][$i+1][$j] . ">";
           $err .= " $tree->[1][$cur][$i+1][$j+1][2] ";
           $err .= "</". $tree->[1][$cur][$i+1][$j] . ">\n";
@@ -292,7 +289,7 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
   }
 
   my @long_descs;
-  for (my $i=3 ; $i < $#{$tree->[1][$cur]} ; $i=$i+4) {
+  for (my $i=3 ; $i < $#{@{$tree}->[1][$cur]} ; $i=$i+4) {
     if ($tree->[1][$cur][$i] =~ /long_desc/) {
       my %long_desc;
       $long_desc{'who'} = $tree->[1][$cur][$i+1][4][2];
@@ -372,20 +369,34 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
      $comp = $default_comp;
   }
 
+  # XXX - why are these arrays??
   my @product;
   my @component;
-  if ((@product = grep { lc($prod) eq lc($_) } @::legal_product) &&
-      (@component = grep { lc($comp) eq lc($_) } @{$::components{$product[0]}}) ) {
-    push (@query, "product");
-    push (@values, SqlQuote($product[0]) );
-    push (@query, "component");
-    push (@values, SqlQuote($component[0]) );
-  } elsif ((@product = grep { lc($default_prod) eq lc($_) } @::legal_product) &&
-      (@component = grep { lc($default_comp) eq lc($_) } @{$::components{$product[0]}}) ) {
-    push (@query, "product");
-    push (@values, SqlQuote($product[0]) );
-    push (@query, "component");
-    push (@values, SqlQuote($component[0]) );
+  my $prod_id;
+  my $comp_id;
+
+  # First, try the given product/component
+  $prod_id = get_product_id($prod);
+  $comp_id = get_component_id($prod_id, $comp) if $prod_id;
+
+  if ($prod_id && $comp_id) {
+      $product[0] = $prod;
+      $component[0] = $comp;
+  } else {
+      # Second, try the defaults
+      $prod_id = get_product_id($default_prod);
+      $comp_id = get_component_id($prod_id, $default_comp) if $prod_id;
+      if ($prod_id && $comp_id) {
+          $product[0] = $default_prod;
+          $component[0] = $default_comp;
+      }
+  }
+
+  if ($prod_id && $comp_id) {
+    push (@query, "product_id");
+    push (@values, $prod_id );
+    push (@query, "component_id");
+    push (@values, $comp_id );
   } else {
     my $subject = "Bug import error: invalid default product or component";
     my $message = "Cannot import these bugs because an invalid default ";
@@ -406,9 +417,9 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
     push (@query, "version");
   } else {
     push (@query, "version");
-    push (@values, SqlQuote($::versions{$product[0]}->[0]));
+    push (@values, SqlQuote(@{$::versions{$product[0]}}->[0]));
     $err .= "Unknown version $bug_fields{'version'} in product $product[0]. ";
-    $err .= "Setting version to \"$::versions{$product[0]}->[0]\".\n";
+    $err .= "Setting version to \"@{$::versions{$product[0]}}->[0]\".\n";
   }
 
   if (defined ($bug_fields{'priority'}) &&
@@ -570,9 +581,10 @@ for (my $k=1 ; $k <= $bugqty ; $k++) {
       push (@values, SqlQuote($qa_contact));
       push (@query, "qa_contact");
     } else {
-      SendSQL("select initialqacontact from components where program=" .
-              SqlQuote($product[0]) .
-              " and value=" . SqlQuote($component[0]) );
+      SendSQL("SELECT initialqacontact FROM components, products " .
+              "WHERE components.product_id = products.id" .
+              " AND products.name = " . SqlQuote($product[0]) .
+              " AND components.name = " . SqlQuote($component[0]) );
       $qa_contact = FetchOneColumn();
       push (@values, SqlQuote(DBname_to_id($qa_contact)) );
       push (@query, "qa_contact");
