@@ -21,13 +21,16 @@
 # Contributor(s): Dawn Endico    <endico@mozilla.org>
 #                 Terry Weissman <terry@mozilla.org>
 
-use diagnostics;
 use strict;
 
-use lib '/usr/share/bugzilla/lib';
+use lib qw(.);
+
+require "CGI.pl";
+
+use vars qw($userid %COOKIE);
 
 use Bug;
-require "CGI.pl";
+
 $::lockcount = 0;
 
 unless ( Param("move-enabled") ) {
@@ -42,7 +45,7 @@ confirm_login();
 sub Log {
     my ($str) = (@_);
     Lock();
-    open(FID, ">>/var/lib/bugzilla/data/maillog") || die "Can't write to /var/lib/bugzilla/data/maillog";
+    open(FID, ">>data/maillog") || die "Can't write to data/maillog";
     print FID time2str("%D %H:%M", time()) . ": $str\n";
     close FID;
     Unlock();
@@ -51,13 +54,13 @@ sub Log {
 sub Lock {
     if ($::lockcount <= 0) {
         $::lockcount = 0;
-        open(LOCKFID, ">>/var/lib/bugzilla/data/maillock") || die "Can't open /var/lib/bugzilla/data/maillock: $!";
+        open(LOCKFID, ">>data/maillock") || die "Can't open data/maillock: $!";
         my $val = flock(LOCKFID,2);
         if (!$val) { # '2' is magic 'exclusive lock' const.
             print "Content-type: text/html\n\n";
             print "Lock failed: $val\n";
         }
-        chmod 0644, "/var/lib/bugzilla/data/maillock";
+        chmod 0666, "data/maillock";
     }
     $::lockcount++;
 }
@@ -94,7 +97,7 @@ unless ($exporter =~ /($movers)/) {
 }
 
 my $xml = "";
-$xml .= Bug::XML_Header( Param("urlbase"), $::param{'version'}, 
+$xml .= Bug::XML_Header( Param("urlbase"), $Bugzilla::Config::VERSION, 
                          Param("maintainer"), $exporter );
 print "<P>\n";
 foreach my $id (split(/:/, $::FORM{'buglist'})) {
@@ -130,7 +133,7 @@ foreach my $id (split(/:/, $::FORM{'buglist'})) {
         "($id,  $exporterid, now(), " . SqlQuote($comment) . ")");
 
     print "<P>Bug $id moved to " . Param("move-to-url") . ".<BR>\n";
-    system("/usr/share/bugzilla/lib/processmail", $id, $exporter);
+    system("./processmail", $id, $exporter);
   }
 }
 print "<P>\n";
@@ -150,7 +153,7 @@ $msg .= "Subject: Moving bug(s) $buglist\n\n";
 $msg .= $xml . "\n";
 
 open(SENDMAIL,
-  "|/usr/lib/sendmail -t -i") ||
+  "|/usr/lib/sendmail -ODeliveryMode=background -t -i") ||
     die "Can't open sendmail";
 print SENDMAIL $msg;
 close SENDMAIL;
