@@ -19,14 +19,26 @@
 # 
 # Contributor(s): Zach Lipton <zach@zachlipton.com>
 #                 Joel Peshkin <bugreport@peshkin.net>
-
+# 
+# Alternatively, the contents of this file may be used under the
+# terms of the GNU General Public License Version 2 or later (the
+# "GPL"), in which case the provisions of the GPL are applicable 
+# instead of those above.  If you wish to allow use of your 
+# version of this file only under the terms of the GPL and not to
+# allow others to use your version of this file under the MPL,
+# indicate your decision by deleting the provisions above and
+# replace them with the notice and other provisions required by
+# the GPL.  If you do not delete the provisions above, a recipient
+# may use your version of this file under either the MPL or the
+# GPL.
+# 
 
 package Support::Files;
 
 # exclude_deps is a hash of arrays listing the files to be excluded
 # if a module is not available
 #
-@additional_files = ('processmail','syncshadowdb');
+@additional_files = ('syncshadowdb','processmail');
 %exclude_deps = (
     'XML::Parser' => ['importxml.pl'],
 );
