@@ -26,11 +26,8 @@
 # assigned to them that has status NEW that has not been touched for
 # more than 7 days.
 
-use diagnostics;
 use strict;
 
-use lib '/usr/share/bugzilla/lib';
-
 require "globals.pl";
 
 ConnectToDatabase();
